const jwtSecret   = '4atikJgznM'
const jwt         = require('jsonwebtoken')

const express           = require('express');
const express_graphql   = require('express-graphql');

const { buildSchema, printSchema, GraphQLString } = require('graphql');
const expand = require('mm-graphql/expand')
const fs     = require('fs')
const uploadPath = `${__dirname}/public/images/`;
const upload  = require('multer')({ dest: uploadPath })


;(async () => {

    const {Savable, slice, getModels} = await require('./models.js')()
    const { jwtGQLAnon, jwtCheck } = require('mm-graphql/jwt')

    let schema = buildSchema(`
        type User {
             _id: ID
             createdAt: String
             login: String
             nick : String
             acl: [String]

             avatar: Media
             chats: [Chat]
        }

        input UserInput {
             _id: ID
             login: String
             nick : String
             password: String
             acl: [String]
             avatar: MediaInput
             chats: [ChatInput]
        }

        type Message {
             _id: ID
             createdAt: String
             owner: User
             chat: Chat
             text: String
             media: [Media]
             replies: [Message]
             replyTo: Message
             forwarded: Message
             forwardWith: [Message]
        }

        input MessageInput {
             _id: ID
             text: String

             chat: ChatInput
             media: [MediaInput]
             replies: [MessageInput]
             replyTo: MessageInput
             forwarded: MessageInput
             forwardWith: [MessageInput]
        }


        type Chat {
            _id: ID
            createdAt: String
            owner: User

            title: String
            members: [User]
            messages: [Message]
            avatar: Media
        }


        input ChatInput {
            _id: ID

            title: String
            members: [UserInput]
            messages: [MessageInput]
        }

        type Media {
            _id: ID,
            createdAt: String

            owner: User
            text: String,
            url: String,
            originalFileName: String,
            type: String


            userAvatar: User
            chatAvatar: [Chat]
            messages: [Message]
        }

        input MediaInput {
            _id: ID,
            text: String,
            userAvatar: UserInput,
            chatAvatar: [ChatInput]
            messages: [MessageInput]
        }
    `);

    schema = expand(schema, {
        login:{
            type: GraphQLString,
            args: {login:    {type: GraphQLString},
                   password: {type: GraphQLString},
            },
            async resolve(root, {login, password}, context, info){
                const Savable =  context.models.Savable 
                if (!login || !password) return null;
                const user    =  await Savable.m.User.findOne({login, password})
                console.log(user, {login, password})
                if (!user)
                    return null;

                const token = jwt.sign({ sub: {id: user._id, login, acl: user.acl}}, jwtSecret); //подписывам токен нашим ключем
                return token
            }
        }
    })
    console.log(printSchema(schema))

    const app = express();
    app.use(require('cors')())
    app.use(express.static('public'));
    app.use('/graphql', express_graphql(jwtGQLAnon({schema, createContext: getModels, graphiql: true, secret: jwtSecret})))


    app.post('/upload', upload.single('media'), async (req, res, next) => {
        let decoded;
        if (decoded = jwtCheck(req, jwtSecret)){
            console.log('SOME UPLOAD', decoded, req.file)

            let {models: {Media }} = await getModels(decoded.sub)
            let media = await Media.fromFileData(req.file)
            res.end(JSON.stringify({_id: media._id, url: media.url}))
        }
        else {
            res.status(503).send('permission denied')
        }
    })

    app.use(express.static('public'));


    let socketPath = "/home/asmer/node_hosts/graphql-chat"
    //let socketPath = 5000
    try {
        fs.unlinkSync(socketPath)
    }
    catch (e) {}
    app.listen(socketPath, () => {
        console.log(`Express GraphQL Server Now Running On ${socketPath}/graphql`);
        fs.chmodSync(socketPath, '777');
    });
})()
